#!/usr/bin/perl -w

sub usage {
	print <<EOF;
pfgen.pl v0.0.1
by Brian Watson, Hardcoders Unlimited

 Usage:
	pfgen.pl filename > filename.inc

 (then, inside your Atari source code):

 org \$xx00
 include "filename.inc"

 Output is to STDOUT only, for now, which is why
 the usage shows a > (redirection) operator.

 You can use a - (hyphen) as a filename, to mean
 `Read from standard input'

 Running pfgen.pl with no arguments or with any
 argument that looks similar to /? or -help will
 get you a copy of this documentation.

 Enjoy,
	B. Watson <atari\@hardcoders.org>

EOF
}


if ((not defined $ARGV[0]) or $ARGV[0] =~ /^[-\/]+(h(elp)?|\?)$/i) {
	&usage;
	exit 0;
}

$res = 1;
$width = 40;
$blank = '0' x 40;
$offset = 0;
$name = "pf";
$maxlines = 192;

while(<>) {
	chomp;
# get rid of comments and skip lines that are comments only
	s/[;#].*//;
	next if /^\s*$/;

# the `res' command
	if (/^\s*res\s*(\d+)/) {
		$res=$1;
		next;
	}

# the `blank' command
	if (/^\s*blank\s*$/) {
		$_ = '';
	}

# the `half' command
	if (/^\s*half\s*/) {
		die "half command must come before playfield data at line $.\n"
			if scalar @lines;
		$half = 1;
		$width = 20;
		next;
	}

	if (/^\s*maxlines\s*(\d+)/) {
		die "maxlines command must come before playfield data at line $.\n"
			if scalar @lines;
		$maxlines = $1;
		next;
	}

	if (/^\s*label\s*(\w+)/) {
		die "label command must come before playfield data at line $.\n"
			if scalar @lines;
		$name = $1;
		next;
	}

	
# the offset command
	if (/^\s*offset\s*(\d+)/) {
		$offset = $1;
		next;
	}

	if (/^\s*fill\s*$/) {
		for (@lines[(scalar @lines)..($maxlines-1)] ) {
			$_ = $blank;
		}
		last;
	}

	s/[^\s0]/1/g;
	s/\s/0/g;
	$_ = ('0' x $offset) . $_ if $offset;
	warn "Line $. too long, truncating to $width pixels" if length($_) > $width;
	$_.=$blank;
	$_ = substr($_,0,40);
	for my $i (1..$res) {
		push @lines,$_;
	}
}

@lines = reverse @lines;
$tlines = scalar @lines;

warn "$tlines lines in playfield\n";
die "Too many lines (should be <=$maxlines)!\n" if $tlines > $maxlines;

print "left_${name}0\n";
for $l (@lines) {
	$_=(scalar reverse substr($l,0,4)) . "0000";
	print " byte %$_\n";
}

print "${name}_bytes = [.-left_${name}0]-1\n\n";
#print qq/ echo "${name}_bytes is",${name}_bytes," when . is",.\n/;

&fix_origin;

print "\nleft_${name}1\n";
for $l (@lines) {
	$_=substr($l,4,8);
	print " byte %$_\n";
}

&fix_origin;


print "\nleft_${name}2\n";
for $l (@lines) {
	$_=scalar reverse substr($l,12,8);
	print " byte %$_\n";
}

&fix_origin;

exit 0 if $half;

print "\nright_${name}0\n";
for $l (@lines) {
	$_=(scalar reverse substr($l,20,4)) . "0000";
	print " byte %$_\n";
}

&fix_origin;

print "\nright_${name}1\n";
for $l (@lines) {
	$_=substr($l,24,8);
	print " byte %$_\n";
}
&fix_origin;


print "\nright_${name}2\n";
for $l (@lines) {
	$_=scalar reverse substr($l,32,8);
	print " byte %$_\n";
}

&fix_origin;
sub fix_origin {
	$pbytes += $tlines;
	if (($tlines+$pbytes) > 256) {
		print " org [>.]*256+256\n";
		$pbytes = 0;
	}
}


__DATA__
res 4
#123456789012345678901234567890123456789
x  xxxxxxxxxxxxxx   x     x  xxxxxxxxxx
x             x  x  x     x        x   x
x  x   xx xxx x  x  x     x   xx   x   x
x x x x   x   x x   x     x  x  x  x  x
x xxx  x  xx  xx    x  x  x  xxxx  xxx
x x x   x x   x x   x  x  x  x  x  x  x
x x x xx  xxx x  x  x x x x  x  x  x   x
x                x  xx   xx            x
xxxxxxxxxxxxxxx  x  x     x  xxxxxxxx  x
